/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import org.junit.rules.ExternalResource;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.junit.exception.MoreThanOneContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoDomainRegisteredException;
import org.ow2.petals.admin.junit.utils.ContainerUtils;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Domain;

/**
 * The PetalsAdministrationApi rule allows creation of a mock implementation of Petals Administration API that is
 * guaranteed to be deleted when the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public static class PetalsAdministrationApi {
 *  &#064;Ruleprotected
 *  public PetalsAdministrationApi petalsAdminApi = new PetalsAdministrationApi(...);
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      petalsAdminApi.registerArtifact(...);
 *      ...
 *  }
 * }
 * </pre>
 * 
 * TODO support multiple domains!
 */
public class PetalsAdministrationApi extends ExternalResource {

    public static final String DOMAIN_NAME = "my-domain";

    /**
     * The information system that will be returned by {@link ContainerAdministration#getSystemInfo()}
     */
    private String systemInfo = null;

    /**
     * The domain definition
     */
    private Domain domain = null;

    /**
     * A domain is registered as <code>null</code>
     */
    private boolean isNullDomain = false;

    private PetalsAdministration singleton;

    @Override
    protected void before() throws Throwable {
        this.initializePetalsAdminApi();
    }

    @Override
    protected void after() {
        try {
            this.destroyPetalsAdminApi();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Initialize the mock implementation of the Petals Administration API
     */
    private void initializePetalsAdminApi() {

        System.setProperty(PetalsAdministrationFactory.PROPERTY_NAME, PetalsAdministrationFactoryMock.class.getName());

        PetalsAdministrationFactory paf = PetalsAdministrationFactory.getInstance();
        if (paf instanceof PetalsAdministrationFactoryMock) {
            ((PetalsAdministrationFactoryMock) paf).registerAdminApi(this);
        } else {
            throw new IllegalArgumentException(
                    "An invalid implementation was selected as Petals Administration Factory: "
                            + paf.getClass().getName());
        }

        this.singleton = paf.newPetalsAdministrationAPI();
    }

    /**
     * Free the mock implementation of the Petals Administration API
     */
    private void destroyPetalsAdminApi() throws Exception {
        if (this.singleton.isConnected()) {
            this.singleton.disconnect();
        }
        System.clearProperty(PetalsAdministrationFactory.PROPERTY_NAME);
        PetalsAdministrationFactory.close();
    }

    /**
     * Register a default domain.
     */
    public PetalsAdministrationApi registerDomain() {
        return this.registerDomain(new Domain(DOMAIN_NAME));
    }

    /**
     * Register the given domain.
     * 
     * @param Domain
     *            The domain to register
     */
    public PetalsAdministrationApi registerDomain(final Domain domain) {
        this.domain = domain;
        this.isNullDomain = false;

        return this;
    }

    /**
     * <p>
     * Register a default domain as <code>null</code>.
     * </p>
     * <p>
     * To use when a connection is required for a command that does not require domain information
     * </p>
     */
    public PetalsAdministrationApi registerNullDomain() {
        this.domain = null;
        this.isNullDomain = true;

        return this;
    }

    Domain getDomain() {
        return this.domain;
    }

    boolean isNullDomain() {
        return isNullDomain;
    }

    /**
     * Register a default container as defined by {@link ContainerUtils#createContainerSample()}.
     * 
     * @throws NoDomainRegisteredException
     *             No domain was previously registered, please use {@link #registerDomain(Domain)} or
     *             {@link #registerDomain()} before this method.
     */
    public PetalsAdministrationApi registerContainer() {
        return this.registerContainer(ContainerUtils.createContainerSample());
    }

    /**
     * Register a container in the current domain.
     * 
     * @param Container
     *            The container to register
     * @throws NoDomainRegisteredException
     *             No domain was previously registered, please use {@link #registerDomain(Domain)} or
     *             {@link #registerDomain()} before this method.
     */
    public PetalsAdministrationApi registerContainer(final Container container) {
        if (this.domain == null) {
            throw new NoDomainRegisteredException();
        } else {
            this.domain.addContainers(container);
        }

        return this;
    }

    /**
     * Register an artifact in the current domain managed with the Petals Administration API Mock.
     * 
     * @param artifact
     *            The artifact to register
     * @throws NoDomainRegisteredException
     *             No domain was previously registered, please use {@link #registerDomain(Domain)} or
     *             {@link #registerDomain()} before this method.
     * @throws NoContainerRegisteredException
     *             No container was previously registered, please use {@link #registerContainer(Container)} or
     *             {@link #registerContainer()} before this method.
     * @throws MoreThanOneContainerRegisteredException
     *             Several containers are registered in the current domain, please use
     *             {@link #registerArtifact(Artifact, Container) to register your artifact for the given container.}
     */
    public PetalsAdministrationApi registerArtifact(final Artifact artifact) {

        if (this.domain == null) {
            throw new NoDomainRegisteredException();
        } else if (this.domain.getContainers() == null || this.domain.getContainers().isEmpty()) {
            throw new NoContainerRegisteredException();
        } else if (this.domain.getContainers().size() > 1) {
            throw new MoreThanOneContainerRegisteredException();
        } else {
            return this.registerArtifact(artifact, this.domain.getContainers().get(0));
        }
    }

    /**
     * Register an artifact in the given container of the current domain managed with the Petals Administration API
     * Mock.
     * 
     * @param artifact
     *            The artifact to register
     * @param container
     *            The container where the artifact must be registered
     */
    public PetalsAdministrationApi registerArtifact(final Artifact artifact, final Container container) {

        if (artifact instanceof SharedLibrary) {
            container.addSharedLibrary((SharedLibrary) artifact);
        } else if (artifact instanceof Component) {
            container.addComponent((Component) artifact);
        } else if (artifact instanceof ServiceAssembly) {
            container.addServiceAssembly((ServiceAssembly) artifact);
        }

        return this;
    }

    /**
     * Register a value returned by {@link ContainerAdministration#getSystemInfo()}
     * 
     * @param systemInfo
     *            The information system that will be returned by {@link ContainerAdministration#getSystemInfo()}
     */
    public void registerSystemInfo(final String systemInfo) {
        this.systemInfo = systemInfo;
    }

    String getSystemInfo() {
        return this.systemInfo;
    }

    /**
     * This is exactly the same as directly using the {@link PetalsAdministrationFactory#getInstance()}
     */
    public PetalsAdministration newPetalsAdministration() {
        return PetalsAdministrationFactory.getInstance().newPetalsAdministrationAPI();
    }

    /**
     * This is valid during the whole test
     */
    public PetalsAdministration getSingleton() {
        return this.singleton;
    }

}